#!/usr/bin/env python

import xmlrpclib
import datetime
import sys, os

SERVER_URL = "http://snipplr.com/xml-rpc.php"
SNIPPLR_SNIPPET_URL = "http://snipplr.com/view.php?id=%(snippet_id)s"
ENV_API_KEY = ""
DEFAULT_SNIPPLR_LANGUAGE = "other"
SNIPPETS_TO_SNIPPLR_LANGUAGE_MAPPER = (
    ("cs", "c-sharp"),
    ("objective-c", "objective-c"),
    ("regex", "regex"),
    ("java", "java"),
    ("lua", "lua"),
    ("actionscript", "actionscript"),
    ("apple-script", "applescript"),
    ("perl", "perl"),
    ("html", "html"),
    ("cold-fusion", "cold-fusion"),
    ("xhtml", "xhtml"),
    ("css", "css"),
    ("xslt", "xslt"),
    ("python", "python"),
    ("javascript", "javascript"),
    ("xml", "xml"),
    ("sql", "sql"),
    ("php", "php"),
    ("ruby", "ruby"),
    ("bash", "bash"),
    ("c", "c"),
    ("cpp", "c-plus-plus"),
    ("lisp", "lisp"),
    ("vb", "vbnet"),
)

snipplr = xmlrpclib.ServerProxy(SERVER_URL)
snipplr_languages = snipplr.languages.list()

api_key = os.environ["SNIPPLR_API_KEY"]

# Snippet name
snippet_name = os.environ["SNIPPET_NAME"]

print "Uploading: ", snippet_name

# Snippet code
snippet_code = os.environ["SNIPPET_SOURCE_CODE"]

# Snippet labels
snippet_labels = os.environ["SNIPPET_LABELS"]
patched_labels = [label.strip().replace(" ", "-") for label in snippet_labels.split(",")]
snipplr_labels = " ".join(patched_labels)

# Snippet highlight
snippet_highlight = os.environ["SNIPPET_SYNTAX_HIGHLIGHT"]
if snippet_highlight not in snipplr_languages.keys():
    if snippet_highlight in dict(SNIPPETS_TO_SNIPPLR_LANGUAGE_MAPPER).keys():
        snippet_highlight = dict(SNIPPETS_TO_SNIPPLR_LANGUAGE_MAPPER)[snippet_highlight]
    else:
        snippet_highlight = DEFAULT_SNIPPLR_LANGUAGE

# Post snippet into snipplr.com
result = snipplr.snippet.post(api_key, snippet_name, snippet_code, snipplr_labels, snippet_highlight)

if result.get('success', "0") == "1":

    url = SNIPPLR_SNIPPET_URL % result
    print "Snippet is uploaded at", url
    
    # Store URL for the next step
    os.environ['SNIPPLR_SNIPPET_URL'] = url

else:

    # ERROR!
    sys.stderr.write("Failed to share snippet")
    sys.exit(1)