#!/usr/bin/env python
# encoding: utf-8
"""
***************************************************************************
Snippet.py
Version 0.1

Created by Snippets on 2010-02-02.
Copyright (c) 2010 Snippetsapp.com. All rights reserved.
***************************************************************************
The MIT License

Copyright (c) 2009 DV Group

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
***************************************************************************
"""
import unittest
import os
import urllib

class Snippet(object):
    """Snippet class"""
    
    LICENSE_APAPCHE2    = "apache2"
    LICENSE_FREEBSD     = "freebsd"
    LICENSE_GPL         = "gpl"
    LICENSE_GPLV3       = "gplv3"
    LICENSE_MIT         = "mit"
    
    LICENCES = {
        LICENSE_APAPCHE2    :"Apache 2",
        LICENSE_FREEBSD     :"FreeBSD",
        LICENSE_GPL         :"GPL",
        LICENSE_GPLV3       :"GPLv3",
        LICENSE_MIT         :"MIT License",
    }
    
    HIGHLIGHT_APPLESCRIPT   = "apple-script"
    HIGHLIGHT_APPLESCRIPT_OBJC = "applescript-objc"
    HIGHLIGHT_ACTIONSCRIPT  = "actionscript"
    HIGHLIGHT_BASH          = "bash"
    HIGHLIGHT_C             = "c"
    HIGHLIGHT_CPP           = "cpp"
    HIGHLIGHT_CSHARP        = "cs"
    HIGHLIGHT_CSS           = "css"
    HIGHLIGHT_COLD_FUSION   = "cold-fusion"
    HIGHLIGHT_ERLANG        = "erlang"
    HIGHLIGHT_JAVASCRIPT    = "javascript"
    HIGHLIGHT_JAVA          = "java"
    HIGHLIGHT_HTML          = "html"
    HIGHLIGHT_LISP          = "lisp"
    HIGHLIGHT_LUA           = "lua"
    HIGHLIGHT_OBJECTIVE_C   = "objective-c"
    HIGHLIGHT_PERL          = "perl"
    HIGHLIGHT_PLAIN_TEXT    = "plain-text"
    HIGHLIGHT_PYTHON        = "python"
    HIGHLIGHT_PHP           = "php"
    HIGHLIGHT_REGEXP        = "regexp"
    HIGHLIGHT_RUBY          = "ruby"
    HIGHLIGHT_VB            = "vb"
    HIGHLIGHT_SQL           = "sql"
    HIGHLIGHT_XML           = "xml"
    HIGHLIGHT_XSLT          = "xslt"
    
    HIGHLIGHTS = {
        HIGHLIGHT_APPLESCRIPT   : "AppleScript",
        HIGHLIGHT_APPLESCRIPT_OBJC : "AppleScriptObjC",
        HIGHLIGHT_ACTIONSCRIPT  : "ActionScript",
        HIGHLIGHT_BASH          : "Bash",
        HIGHLIGHT_C             : "C",
        HIGHLIGHT_CPP           : "C++",
        HIGHLIGHT_CSHARP        : "C#",
        HIGHLIGHT_CSS           : "CSS",
        HIGHLIGHT_COLD_FUSION   : "ColdFusion",
        HIGHLIGHT_ERLANG        : "Erlang",
        HIGHLIGHT_JAVASCRIPT    : "JavaScript",
        HIGHLIGHT_JAVA          : "Java",
        HIGHLIGHT_HTML          : "HTML",
        HIGHLIGHT_LISP          : "Lisp",
        HIGHLIGHT_LUA           : "Lua",
        HIGHLIGHT_OBJECTIVE_C   : "Objective-C",
        HIGHLIGHT_PERL          : "Perl",
        HIGHLIGHT_PLAIN_TEXT    : "Plain Text",
        HIGHLIGHT_PYTHON        : "Python",
        HIGHLIGHT_PHP           : "PHP",
        HIGHLIGHT_REGEXP        : "Regular Expression",
        HIGHLIGHT_RUBY          : "Ruby",
        HIGHLIGHT_VB            : "VB.NET",
        HIGHLIGHT_SQL           : "SQL",
        HIGHLIGHT_XML           : "XML",
        HIGHLIGHT_XSLT          : "XSLT",
    }
    HIGHLIGHT_EXTENSIONS = {
        HIGHLIGHT_APPLESCRIPT   : "script",
        HIGHLIGHT_APPLESCRIPT_OBJC : "script",
        HIGHLIGHT_ACTIONSCRIPT  : "as",
        HIGHLIGHT_BASH          : "sh",
        HIGHLIGHT_C             : "c",
        HIGHLIGHT_CPP           : "cpp",
        HIGHLIGHT_CSHARP        : "csharp",
        HIGHLIGHT_CSS           : "css",
        HIGHLIGHT_COLD_FUSION   : "cf",
        HIGHLIGHT_ERLANG        : "erl",
        HIGHLIGHT_JAVASCRIPT    : "js",
        HIGHLIGHT_JAVA          : "java",
        HIGHLIGHT_HTML          : "html",
        HIGHLIGHT_LISP          : "lisp",
        HIGHLIGHT_LUA           : "lua",
        HIGHLIGHT_OBJECTIVE_C   : "c",
        HIGHLIGHT_PERL          : "pl",
        HIGHLIGHT_PLAIN_TEXT    : "txt",
        HIGHLIGHT_PYTHON        : "py",
        HIGHLIGHT_PHP           : "php",
        HIGHLIGHT_REGEXP        : "regexp",
        HIGHLIGHT_RUBY          : "rb",
        HIGHLIGHT_VB            : "vb",
        HIGHLIGHT_SQL           : "sql",
        HIGHLIGHT_XML           : "xml",
        HIGHLIGHT_XSLT          : "xslt",
    }
    
    ENV_CODE                = "SNIPPET_SOURCE_CODE"
    ENV_NAME                = "SNIPPET_NAME"
    ENV_DESCRIPTION         = "SNIPPET_DESCRIPTION"
    ENV_LABELS              = "SNIPPET_LABELS"
    ENV_HIGHLIGHT           = "SNIPPET_SYNTAX_HIGHLIGHT"
    ENV_RELATED_URL         = "SNIPPET_RELATED_URL"
    ENV_AUTHOR_NAME         = "SNIPPET_AUTHOR"
    ENV_EXPORT_FILENAME     = "SNIPPET_EXPORT_FILENAME"
    ENV_LICENSE             = "SNIPPET_LICENSE"
    ENV_INCLUDE_LICENSE     = "SNIPPET_INCLUDE_LICENSE"
    
    DEFAULT_CODE            = ""
    DEFAULT_NAME            = "Snippet"
    DEFAULT_DESCRIPTION     = ""
    DEFAULT_LABELS          = []
    DEFAULT_HIGHLIGHT       = HIGHLIGHT_PLAIN_TEXT
    DEFAULT_RELATED_URL     = ""
    DEFAULT_AUTHOR_NAME     = ""
    DEFAULT_EXPORT_FILENAME = ""
    DEFAULT_LICENSE         = LICENCES.items()[0][0]
    DEFAULT_INCLUDE_LICENSE = False
    
    def __init__(self):
        super(Snippet, self).__init__()
        self.code           = Snippet.DEFAULT_CODE
        self.name           = Snippet.DEFAULT_NAME
        self.description    = Snippet.DEFAULT_DESCRIPTION
        self.labels         = Snippet.DEFAULT_LABELS
        self.highlight      = Snippet.DEFAULT_HIGHLIGHT
        self.related_url    = Snippet.DEFAULT_RELATED_URL
        self.author_name    = Snippet.DEFAULT_AUTHOR_NAME
        self.export_filename= Snippet.DEFAULT_EXPORT_FILENAME
        self.license        = Snippet.DEFAULT_LICENSE
        self.include_license= Snippet.DEFAULT_INCLUDE_LICENSE
    
    def __unicode__(self):
        return "<Snippet '%s'>" % self.name
    
    def __str__(self):
        return self.__unicode__()
    
    def __setattr__(self, key, value):
        if key == "labels":
            if type(value) == list:
                pass
            elif type(value) == str:
                value = [label.strip() for label in value.split(",")]
            elif type(value) == int:
                value = [str(value)]
        elif key == "labels":
            if value not in Snippet.HIGHLIGHTS:
                value = Snippet.DEFAULT_HIGHLIGHT
        elif key == "include_license":
            if type(value) != bool:
                value = bool(value)
        
        return super(Snippet, self).__setattr__(key, value)
    
    def share(self, factory):
        url = factory.share(self)
        return url
    def set_attrs_from_envvars(self):
        self.name               = os.environ.get(Snippet.ENV_NAME,              Snippet.DEFAULT_NAME)
        self.code               = os.environ.get(Snippet.ENV_CODE,              Snippet.DEFAULT_CODE)
        self.description        = os.environ.get(Snippet.ENV_DESCRIPTION,       Snippet.DEFAULT_DESCRIPTION)
        self.labels             = os.environ.get(Snippet.ENV_LABELS,            Snippet.DEFAULT_LABELS)
        self.highlight          = os.environ.get(Snippet.ENV_HIGHLIGHT,         Snippet.DEFAULT_HIGHLIGHT)
        self.related_url        = os.environ.get(Snippet.ENV_RELATED_URL,       Snippet.DEFAULT_RELATED_URL)
        self.author_name        = os.environ.get(Snippet.ENV_AUTHOR_NAME,       Snippet.DEFAULT_AUTHOR_NAME)
        self.export_filename    = os.environ.get(Snippet.ENV_EXPORT_FILENAME,   Snippet.DEFAULT_EXPORT_FILENAME)
        self.license            = os.environ.get(Snippet.ENV_LICENSE,           Snippet.DEFAULT_LICENSE)
        self.include_license    = os.environ.get(Snippet.ENV_INCLUDE_LICENSE,   Snippet.DEFAULT_INCLUDE_LICENSE)
    
    def get_snippet_url(self):
        url = "snippet:add?code=%s" % urllib.quote(self.code)
        attrs = {}
        if self.name:
            attrs['name'] = self.name
        
        if self.description:
            attrs['notes'] = self.description
        
        if self.labels:
            attrs['labels'] = ",".join(self.labels)
        
        if self.description:
            attrs['highlight'] = self.highlight
        
        if self.related_url:
            attrs['relatedurl'] = self.related_url
        
        if self.author_name:
            attrs['author'] = self.author_name
        
        if self.license:
            attrs['license'] = self.license
        
        if self.include_license:
            attrs['export_license'] = "1"
        else:
            attrs['export_license'] = "0"
        
        attrs_string = "&".join(["%s=%s" % (k,urllib.quote(v)) for k,v in attrs.items()])
        return "%s&%s" % (url,attrs_string)
        
        

class TestSnippet(unittest.TestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def test_defaults(self):
        snip = Snippet()
        snip.set_attrs_from_envvars()
        self.assertEqual(snip.name,             Snippet.DEFAULT_NAME)
        self.assertEqual(snip.description,      Snippet.DEFAULT_DESCRIPTION)
        self.assertEqual(snip.labels,           Snippet.DEFAULT_LABELS)
        self.assertEqual(snip.highlight,        Snippet.DEFAULT_HIGHLIGHT)
        self.assertEqual(snip.related_url,      Snippet.DEFAULT_RELATED_URL)
        self.assertEqual(snip.author_name,      Snippet.DEFAULT_AUTHOR_NAME)
        self.assertEqual(snip.export_filename,  Snippet.DEFAULT_EXPORT_FILENAME)
        self.assertEqual(snip.license,          Snippet.DEFAULT_LICENSE)
        self.assertEqual(snip.include_license,  Snippet.DEFAULT_INCLUDE_LICENSE)
    
    def test_defaults2(self):
        snip = Snippet()
        os.environ[Snippet.ENV_NAME] = "test name"
        os.environ[Snippet.ENV_DESCRIPTION] = "test description"
        os.environ[Snippet.ENV_LABELS] = "test, labels"
        os.environ[Snippet.ENV_HIGHLIGHT] = Snippet.HIGHLIGHT_PYTHON
        os.environ[Snippet.ENV_RELATED_URL] = "http://www.snippetsapp.com/egg/"
        os.environ[Snippet.ENV_AUTHOR_NAME] = "Prudnikov Vladimir"
        os.environ[Snippet.ENV_EXPORT_FILENAME] = "test_defaults2.py"
        os.environ[Snippet.ENV_LICENSE] = "test_defaults2.py"
        os.environ[Snippet.ENV_INCLUDE_LICENSE] = "1"
        
        snip.set_attrs_from_envvars()
        self.assertNotEqual(snip.name,              Snippet.DEFAULT_NAME)
        self.assertNotEqual(snip.description,       Snippet.DEFAULT_DESCRIPTION)
        self.assertNotEqual(snip.labels,            Snippet.DEFAULT_LABELS)
        self.assertNotEqual(snip.highlight,         Snippet.DEFAULT_HIGHLIGHT)
        self.assertNotEqual(snip.related_url,       Snippet.DEFAULT_RELATED_URL)
        self.assertNotEqual(snip.author_name,       Snippet.DEFAULT_AUTHOR_NAME)
        self.assertNotEqual(snip.export_filename,   Snippet.DEFAULT_EXPORT_FILENAME)
        self.assertNotEqual(snip.license,           Snippet.DEFAULT_LICENSE)
        self.assertNotEqual(snip.include_license,   Snippet.DEFAULT_INCLUDE_LICENSE)
    
    def test_name(self):
        os.environ[Snippet.ENV_NAME] = "123Name"
        snip = Snippet()
        snip.set_attrs_from_envvars()
        self.assertEqual(snip.name, "123Name")
    
    def test_highlight(self):
        os.environ[Snippet.ENV_HIGHLIGHT] = "somethingunknown"
        snip = Snippet()
        snip.set_attrs_from_envvars()
        self.assertNotEqual(snip.highlight, Snippet.DEFAULT_HIGHLIGHT)
        
        os.environ[Snippet.ENV_HIGHLIGHT] = Snippet.HIGHLIGHT_PYTHON
        snip = Snippet()
        snip.set_attrs_from_envvars()
        self.assertEqual(snip.highlight, Snippet.HIGHLIGHT_PYTHON)
    
    def test_labels(self):
        label_tests = (
            ("label1 label2", ["label1 label2"]),
            ("label1, label2", ["label1", "label2"]),
            ("   label1  , label2   ", ["label1", "label2"]),
        )
        for (in_value, out_value) in label_tests:
            os.environ['SNIPPET_LABELS'] = in_value
            snip = Snippet()
            snip.set_attrs_from_envvars()
            self.assertEqual(snip.labels, out_value)
    
    

if __name__=="__main__":
    sn = Snippet()
    print sn.get_snippet_url()
    unittest.main()
