#!/usr/bin/env python
import urllib, urllib2, os, sys
import json
import snippet

POST_URL = 'http://api.snipt.org/makeSnipt/'
crawler_user_agent = 'Snippets/%s (+http://www.snippetsapp.com/scrippets-crawler.html) Scrippet %s crawler' % (
    os.environ.get("SNIPPETS_VERSION", "Unknown version"),
    os.environ.get("SCRIPPET_IDENTIFIER", "Unknown ID"),
)
HIGHLIGHT_MAPPING = {
    'apple-script':'applescript',
    'applescript-objc':'applescript',
    'cs':'csharp',
    'cold-fusion':'text',
    'erlang':'text',
    'html':'html4strict',
    'objective-c':'objc',
    'plain-text':'text',
    'regexp':'reg',
    'xslt':'xml'
}

snip = snippet.Snippet()
snip.set_attrs_from_envvars()

_is_private = os.environ.get("SNIPTORG_PRIVATE_FLAG","false")
if _is_private == 'true':
    is_private = "1"
else:
    is_private = "0"

_show_nums = os.environ.get("SNIPTORG_PRIVATE_FLAG","false")
if _show_nums == 'true':
    show_nums = "1"
else:
    show_nums = "0"

if snip.highlight in HIGHLIGHT_MAPPING.keys():
    lang = HIGHLIGHT_MAPPING[snip.highlight]
else:
    lang = snip.highlight

post_data = {
    "content":snip.code,          # Snipt content
    "language":lang,          # Language of the snipt
    "title":snip.name,          # (optional) - Snipt title
    "isPrivate":is_private,    # (optional) - If this Snipt is private or not
    "showNums":show_nums,
    "token":os.environ.get("SNIPTORG_API_KEY"), # (optional) - Username of SniptBox of which to add this Snipt
}
request = urllib2.Request(POST_URL,urllib.urlencode(post_data))
request.add_header('User-Agent', crawler_user_agent)

sys.stdout.write("Uploading '%s'\n" % snip.name)

handler = urllib2.urlopen(request)
response = handler.read()

response_data = json.loads(response)
if "error" in response_data.keys():
    error_message = "Failed to submit snippet. Error is %s (Error code: #%s)\n\n" % (response_data.get("message"), response_data.get("code", "Unknown"))
    sys.stderr.write(error_message)
    sys.exit(1)
else:
    snippet_url = "http://snipt.org/%s" % response_data['ID']
    sys.stdout.write("Snippet is successfully uploaded at %s\n\n" % snippet_url)
    os.environ['SNIPTORG_SNIPPET_URL'] = snippet_url
